/*******************************************************************************//**
 * \file sangoma_transcode.h
 * \brief Sangoma Transcoding library for Vocallo/Sangoma Transcoding Board
 * \brief Provides User a Unified/OS Agnostic API to Sangoma Transcoding Board
 *
 * Author(s):   Nenad Corbic <ncorbic@sangoma.com>
 *
 * Copyright:	(c) 2009-2009 Nenad Corbic <ncorbic@sangoma.com>
*/

#ifndef __LIB_SANGOMA_TRANSCODE__
#define __LIB_SANGOMA_TRANSCODE__

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <stddef.h>
#include <stdint.h>
#include <stdarg.h>
#include <pthread.h>
#include "sng_tc_version.h"


enum sngtc_codec_definition {
	SNGTC_CODEC_MIN_ID	= 0,

	SNGTC_CODEC_PCMU	= 1,
	SNGTC_CODEC_PCMA	= 2,
	SNGTC_CODEC_L16_1	= 3,
	SNGTC_CODEC_L16_2	= 4,
	SNGTC_CODEC_G729AB	= 5,
	SNGTC_CODEC_G722	= 6,
	SNGTC_CODEC_GSM_FR	= 7,
	SNGTC_CODEC_GSM_EFR	= 8,
	SNGTC_CODEC_AMR		= 9,
	SNGTC_CODEC_AMR_WB	= 10,
	SNGTC_CODEC_G726_16	= 11,
	SNGTC_CODEC_G726_24	= 12,
	SNGTC_CODEC_G726_32	= 13,
	SNGTC_CODEC_G726_40	= 14,
	SNGTC_CODEC_ILBC	= 15,

	SNGTC_CODEC_MAX_ID, 
};

#define SNGTC_CODEC_ID_DECODE(id) \
		(id==SNGTC_CODEC_PCMU) ? "SNGTC_CODEC_PCMU" : \
		(id==SNGTC_CODEC_PCMA) ? "SNGTC_CODEC_PCMA" : \
		(id==SNGTC_CODEC_L16_1) ? "SNGTC_CODEC_L16_1" : \
		(id==SNGTC_CODEC_L16_2) ? "SNGTC_CODEC_L16_2" : \
		(id==SNGTC_CODEC_G729AB) ? "SNGTC_CODEC_G729AB" : \
		(id==SNGTC_CODEC_G722) ? "SNGTC_CODEC_G722" : \
		(id==SNGTC_CODEC_GSM_FR) ? "SNGTC_CODEC_GSM_FR" : \
		(id==SNGTC_CODEC_GSM_EFR) ? "SNGTC_CODEC_GSM_EFR" : \
		(id==SNGTC_CODEC_AMR) ? "SNGTC_CODEC_AMR" : \
		(id==SNGTC_CODEC_G726_32) ? "SNGTC_CODEC_G726_32" : \
		(id==SNGTC_CODEC_ILBC) ? "SNGTC_CODEC_ILIBC" : "Unknown"




enum sngtc_rpt_session_type
{
	SNGTC_RTP_SESSION_A,
	SNGTC_RTP_SESSION_B
};

struct sngtc_codec_reply;



typedef struct sngtc_codec_cfg
{
	uint32_t 	codec_id;
	uint32_t	ms;

	uint32_t	host_ip;
	uint32_t	host_netmask;
	uint32_t	host_udp_port;

} sngtc_codec_cfg_t, sngtc_codec_request_leg_t;

typedef struct  sngtc_codec_net_cfg
{
	uint32_t	codec_ip;
	uint32_t	codec_netmask;
	uint32_t	codec_udp_port;

	uint32_t	host_ip;
	uint32_t	host_netmask;
	uint32_t	host_udp_port;

	uint32_t	iana_code;

}sngtc_codec_net_cfg_t, sngtc_codec_reply_leg_t;


typedef struct sngtc_codec_reply
{
	void 		*codec_module_session;
	void		*codec_rtp_session;

	sngtc_codec_reply_leg_t a;
	sngtc_codec_reply_leg_t b;

	void		*tx_fd;
	uint32_t	tx_iana;

	void		*rx_fd;
	uint32_t	rx_iana;

} sngtc_codec_reply_t;
 

typedef struct sngtc_codec_request
{
	void *usr_priv;

	uint8_t		rtcp_enable;

	sngtc_codec_request_leg_t a;
	sngtc_codec_request_leg_t b;

} sngtc_codec_request_t;


#define SNGTC_MAX_MAC_RANGE 		32
#define SNGTC_MAX_HOST_VOCALLO_NIC 	32
#define SNGTC_MAX_ETH_SZ			6

typedef struct sngtc_eth_mac
{
	char mac_str[18];
	uint8_t mac[6];
}sngtc_eth_mac_t;

typedef struct sngtc_ip
{
	char ip_str[25];
	uint32_t ip;
}sngtc_ip_t;


typedef struct sngtc_vocallo_map {
	uint8_t init;

	sngtc_eth_mac_t v_mac;
	sngtc_ip_t 		v_ip;
	uint32_t		v_eth_port;

	uint32_t		tdm_timestamp_reg;
	uint32_t		error;
}sngtc_vocallo_map_t;

#define MAX_IP_STR_SIZE 100
typedef struct sngtc_nic_vocallo_map{
	uint8_t init;
	sngtc_eth_mac_t nic_mac;
	sngtc_ip_t nic_ip;
	sngtc_ip_t nic_netmask;

	sngtc_ip_t voc_base_ip;
	sngtc_ip_t voc_base_netmask;
	int        voc_base_udp_prot;

	int voc_mods_sz;
	sngtc_vocallo_map_t voc_map[SNGTC_MAX_MAC_RANGE];
}sngtc_nic_vocallo_map_t;


/*!
  \struct sngtc_host_nic_vocallo_cfg
  \brief Sangoma transcode init structure used to configure the transcode with initial parmeters.
  \typedef sngtc_host_vocallo_cfg
  \brief Sangoma transcode init structure used to configure the transcode with initial parmeters.
*/
typedef struct sngtc_host_nic_vocallo_cfg
{
	uint32_t 	vocallo_base_udp_port;
	uint32_t	vocallo_ip;
	uint32_t	vocallo_ip_netmask;

	uint32_t	host_ip;
	uint32_t	host_ip_netmask;

	sngtc_eth_mac_t	host_mac;

	/* Optional to select a speicifc vocallo */
	sngtc_eth_mac_t	vocallo_macs[SNGTC_MAX_MAC_RANGE];
	uint32_t		vocallo_macs_sz;

	uint8_t		silence_suppression;

} sngtc_host_nic_vocallo_cfg_t;


/*!
  \struct sngtc_init_cfg
  \brief Sangoma transcode init structure used to configure the transcode with initial parmeters.
  \typedef sngtc_init_cfg_t
  \brief Sangoma transcode init structure used to configure the transcode with initial parmeters.
*/ 

typedef struct sngtc_init_cfg
{
	uint32_t						host_nic_vocallo_sz;
	sngtc_host_nic_vocallo_cfg_t	host_nic_vocallo_cfg[SNGTC_MAX_HOST_VOCALLO_NIC];
	
	int (*log)( int level, char *fmt, ...);
	int (*create_rtp)(void *usr_priv, sngtc_codec_request_leg_t *codec_reg_leg, sngtc_codec_reply_leg_t* codec_reply_leg, void **rtp_fd);
	int (*destroy_rtp)(void *usr_priv, void *fd);

} sngtc_init_cfg_t;



/***************************
 Statistics
***************************/
/*!
	\struct sngtc_rtcp_sender_stats
	\brief RTCP Sender Statistics
	\typedef sngtc_rtcp_sender_stats_t
*/
typedef struct sngtc_rtcp_sender_stats
{
	uint32_t SentPktCnt[2];
	uint32_t SentByteCnt[2];
	uint32_t NtpTimestamp[2];
	uint32_t RtpTimestamp;
	uint32_t SenderSsrc;
	uint32_t SenderStartTime;
	uint32_t SenderReportCnt;
	uint32_t SenderState;
	uint32_t LastSenderReportTime;
	uint32_t SsrcChangeCnt;
} sngtc_rtcp_sender_stats_t;

/*!
	\struct sngtc_rtcp_receiver_stats
	\brief RTCP Receiver Statistics
	\typedef sngtc_rtcp_receiver_stats_t
*/
typedef struct sngtc_rtcp_receiver_stats
{
	uint32_t ulHighestSeqNumber;
	uint32_t ulCumulativeLostCnt;
	uint32_t ulFractionLost;
	uint32_t ulInterArrivalJitter;
	uint32_t ulMaxFractionLost;
	uint32_t ulMaxInterArrivalJitter;
	uint32_t ulAverageFractionLost;
	uint32_t ulAverageInterArrivalJitter;
	uint32_t ulReceiverStartTime;
	uint32_t ulReceiverReportCnt;
	uint32_t ulLastReceiverReportTime;
	uint32_t ulRemoteSenderSsrc;
	uint32_t ulAlarmRxThreshold;
	uint32_t ulAlarmRxThresholdCnt;
} sngtc_rtcp_receiver_stats_t;

/*!
	\struct sngtc_rtp_global_stats
	\brief RTP Global Statistics
	\typedef sngtc_rtp_global_stats_t
*/
typedef struct sngtc_rtp_global_stats
{
	uint32_t NumMember;
	uint32_t NumSender;
	uint32_t SessionStartTime;
	uint32_t RxByePktCnt;
	uint32_t RxSenderJoinCnt;
	uint32_t RxMemberJoinCnt;
	uint32_t ErrCollisionCnt;
	uint32_t ErrLoopCnt;
	uint32_t ErrRtcpPktInvalidCnt;
} sngtc_rtp_global_stats_t ;

/*!
	\struct sngtc_rtp_local_stats
	\brief Local RTP Statistics
	\typedef sngtc_rtp_local_stats_t
*/
typedef struct sngtc_rtp_local_stats
{
	/* Local Statistics */
	uint32_t ReceivedPktCnt[2];
	uint32_t ReceivedByteCnt[2];
	sngtc_rtcp_sender_stats_t SenderStats;
	sngtc_rtcp_receiver_stats_t ReceiverStats;
} sngtc_rtp_local_stats_t;


/*!
	\struct sngtc_rtp_remote_stats
	\brief Remote RTP Statistics
	\typedef sngtc_rtp_remote_stats_t
*/
typedef struct sngtc_rtp_remote_stats
{
	uint32_t RoundTripTime;
	uint32_t MaxRoundTripTime;
	uint32_t AverageRoundTripTime;
	sngtc_rtcp_sender_stats_t SenderStats;
	sngtc_rtcp_receiver_stats_t ReceiverStats;
} sngtc_rtp_remote_stats_t;

/*!
	\struct sngtc_rtp_stats
	\brief Full RTP Statistics both Local & Remote
	\typedef sngtc_rtp_stats_t
*/
typedef struct sngtc_rtp_stats
{
	sngtc_rtp_global_stats_t stats;
	sngtc_rtp_local_stats_t	 local_stats;
	sngtc_rtp_remote_stats_t remote_stats;

} sngtc_rtp_stats_t;


#define SNGTC_NIPV4(addr) \
	((unsigned char *)&addr)[3], \
	((unsigned char *)&addr)[2], \
	((unsigned char *)&addr)[1], \
	((unsigned char *)&addr)[0]

#define SNGTC_NIPV4_FMT "%u.%u.%u.%u"

/*!
  \enum SNGTC_LOGLEVEL_DEBUG
  \enum SNGTC_LOGLEVEL_WARN
  \enum SNGTC_LOGLEVEL_INFO
  \enum SNGTC_LOGLEVEL_STATS
  \enum SNGTC_LOGLEVEL_ERROR
  \enum SNGTC_LOGLEVEL_CRIT
*/
enum {
	SNGTC_LOGLEVEL_DEBUG=1,
	SNGTC_LOGLEVEL_WARN,
	SNGTC_LOGLEVEL_INFO,
	SNGTC_LOGLEVEL_STATS,
	SNGTC_LOGLEVEL_ERROR,
	SNGTC_LOGLEVEL_CRIT,
	SNGTC_LOGLEVEL_PUBLIC=100,
};

#define SNGTC_LOGLEVEL_DECODE(level) \
		((level==SNGTC_LOGLEVEL_DEBUG) ? "SNGTC_DEBUG" : \
		(level==SNGTC_LOGLEVEL_WARN) ? "SNGTC_WARN" : \
		(level==SNGTC_LOGLEVEL_INFO) ? "SNGTC_INFO" : \
		(level==SNGTC_LOGLEVEL_STATS) ? "SNGTC_STATS" : \
		(level==SNGTC_LOGLEVEL_ERROR) ? "SNGTC_ERROR" : \
		(level==SNGTC_LOGLEVEL_CRIT) ? "SNGTC_CRIT" : "Unkown")

int sngtc_get_nic_vocallo_map(sngtc_init_cfg_t *cfg, sngtc_nic_vocallo_map_t **nic_vocallo_map, int *nic_voc_map_idx);
int sngtc_detect_init_modules(sngtc_init_cfg_t *cfg, int *detected);
int sngtc_activate_modules(sngtc_init_cfg_t *cfg, int *activated);
int sngtc_deactivate_modules(void);

int sngtc_codec_ipv4_str_hex(const char *ip_str, uint32_t *ip_addr);
int sngtc_codec_ipv4_hex_to_str(uint32_t ip_addr, char *ip_str);
int sngtc_rtp_get_status(void *codec_obj, int rtp_a, sngtc_rtp_stats_t *rtp_stats);

int sngtc_create_transcoding_session (sngtc_codec_request_t *codec_req, sngtc_codec_reply_t *codec_reply, int start_module);
int sngtc_destroy_transcoding_session (void *codec_obj);
int sngtc_all_free_sanity_check(void);
int sngtc_detected_vocallo_modules(void);

void *sngtc_module_get_instance(int instance_idx);
int sngtc_module_reset(void *module_obj);
int sngtc_module_restore_session(void *module_obj);
int sngtc_module_create_host(void *module_obj);
int sngtc_module_write_file(void *module, char *local_file, char *remote_file, int progress);
char *sngtc_module_mac(void *module_obj);
int sngtc_set_iana_code_based_on_codec_id(uint32_t codec_id, uint32_t iana);
void sngtc_mac_addr_to_str(char *mac_str, uint8_t mac[SNGTC_MAX_ETH_SZ]);
#endif










